// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: internal/scalibrextract/language/osv/osvscannerjson/proto/metadata.proto

package proto

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type OSVScannerJsonMetadata struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Ecosystem     string                 `protobuf:"bytes,1,opt,name=ecosystem,proto3" json:"ecosystem,omitempty"`
	SourceInfo    *SourceInfo            `protobuf:"bytes,2,opt,name=source_info,json=sourceInfo,proto3" json:"source_info,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OSVScannerJsonMetadata) Reset() {
	*x = OSVScannerJsonMetadata{}
	mi := &file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OSVScannerJsonMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OSVScannerJsonMetadata) ProtoMessage() {}

func (x *OSVScannerJsonMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OSVScannerJsonMetadata.ProtoReflect.Descriptor instead.
func (*OSVScannerJsonMetadata) Descriptor() ([]byte, []int) {
	return file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *OSVScannerJsonMetadata) GetEcosystem() string {
	if x != nil {
		return x.Ecosystem
	}
	return ""
}

func (x *OSVScannerJsonMetadata) GetSourceInfo() *SourceInfo {
	if x != nil {
		return x.SourceInfo
	}
	return nil
}

type SourceInfo struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Path          string                 `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	Type          string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SourceInfo) Reset() {
	*x = SourceInfo{}
	mi := &file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SourceInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SourceInfo) ProtoMessage() {}

func (x *SourceInfo) ProtoReflect() protoreflect.Message {
	mi := &file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SourceInfo.ProtoReflect.Descriptor instead.
func (*SourceInfo) Descriptor() ([]byte, []int) {
	return file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescGZIP(), []int{1}
}

func (x *SourceInfo) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *SourceInfo) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

var File_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto protoreflect.FileDescriptor

const file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDesc = "" +
	"\n" +
	"Hinternal/scalibrextract/language/osv/osvscannerjson/proto/metadata.proto\x12\x0eosvscannerjson\"s\n" +
	"\x16OSVScannerJsonMetadata\x12\x1c\n" +
	"\tecosystem\x18\x01 \x01(\tR\tecosystem\x12;\n" +
	"\vsource_info\x18\x02 \x01(\v2\x1a.osvscannerjson.SourceInfoR\n" +
	"sourceInfo\"4\n" +
	"\n" +
	"SourceInfo\x12\x12\n" +
	"\x04path\x18\x01 \x01(\tR\x04path\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04typeB\\ZZgithub.com/google/osv-scanner/v2/internal/scalibrextract/language/osv/osvscannerjson/protob\x06proto3"

var (
	file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescOnce sync.Once
	file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescData []byte
)

func file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescGZIP() []byte {
	file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescOnce.Do(func() {
		file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDesc), len(file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDesc)))
	})
	return file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDescData
}

var file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_goTypes = []any{
	(*OSVScannerJsonMetadata)(nil), // 0: osvscannerjson.OSVScannerJsonMetadata
	(*SourceInfo)(nil),             // 1: osvscannerjson.SourceInfo
}
var file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_depIdxs = []int32{
	1, // 0: osvscannerjson.OSVScannerJsonMetadata.source_info:type_name -> osvscannerjson.SourceInfo
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_init() }
func file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_init() {
	if File_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDesc), len(file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_goTypes,
		DependencyIndexes: file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_depIdxs,
		MessageInfos:      file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_msgTypes,
	}.Build()
	File_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto = out.File
	file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_goTypes = nil
	file_internal_scalibrextract_language_osv_osvscannerjson_proto_metadata_proto_depIdxs = nil
}
