// Code generated by gen.go. DO NOT EDIT.

package spdx

var IDs = map[string]bool{
	"0bsd":                                 true,
	"3d-slicer-1.0":                        true,
	"aal":                                  true,
	"abstyles":                             true,
	"adacore-doc":                          true,
	"adobe-2006":                           true,
	"adobe-display-postscript":             true,
	"adobe-glyph":                          true,
	"adobe-utopia":                         true,
	"adsl":                                 true,
	"advanced-cryptics-dictionary":         true,
	"afl-1.1":                              true,
	"afl-1.2":                              true,
	"afl-2.0":                              true,
	"afl-2.1":                              true,
	"afl-3.0":                              true,
	"afmparse":                             true,
	"agpl-1.0":                             true,
	"agpl-1.0-only":                        true,
	"agpl-1.0-or-later":                    true,
	"agpl-3.0":                             true,
	"agpl-3.0-only":                        true,
	"agpl-3.0-or-later":                    true,
	"aladdin":                              true,
	"alglib-documentation":                 true,
	"amd-newlib":                           true,
	"amdplpa":                              true,
	"aml":                                  true,
	"aml-glslang":                          true,
	"ampas":                                true,
	"antlr-pd":                             true,
	"antlr-pd-fallback":                    true,
	"any-osi":                              true,
	"any-osi-perl-modules":                 true,
	"apache-1.0":                           true,
	"apache-1.1":                           true,
	"apache-2.0":                           true,
	"apafml":                               true,
	"apl-1.0":                              true,
	"app-s2p":                              true,
	"apsl-1.0":                             true,
	"apsl-1.1":                             true,
	"apsl-1.2":                             true,
	"apsl-2.0":                             true,
	"arphic-1999":                          true,
	"artistic-1.0":                         true,
	"artistic-1.0-cl8":                     true,
	"artistic-1.0-perl":                    true,
	"artistic-2.0":                         true,
	"artistic-dist":                        true,
	"aspell-ru":                            true,
	"aswf-digital-assets-1.0":              true,
	"aswf-digital-assets-1.1":              true,
	"baekmuk":                              true,
	"bahyph":                               true,
	"barr":                                 true,
	"bcrypt-solar-designer":                true,
	"beerware":                             true,
	"bitstream-charter":                    true,
	"bitstream-vera":                       true,
	"bittorrent-1.0":                       true,
	"bittorrent-1.1":                       true,
	"blessing":                             true,
	"blueoak-1.0.0":                        true,
	"boehm-gc":                             true,
	"boehm-gc-without-fee":                 true,
	"bola-1.1":                             true,
	"borceux":                              true,
	"brian-gladman-2-clause":               true,
	"brian-gladman-3-clause":               true,
	"brian-gladman-3-clause-no-conversion": true,
	"bsd-1-clause":                         true,
	"bsd-2-clause":                         true,
	"bsd-2-clause-darwin":                  true,
	"bsd-2-clause-first-lines":             true,
	"bsd-2-clause-freebsd":                 true,
	"bsd-2-clause-netbsd":                  true,
	"bsd-2-clause-patent":                  true,
	"bsd-2-clause-pkgconf-disclaimer":      true,
	"bsd-2-clause-views":                   true,
	"bsd-3-clause":                         true,
	"bsd-3-clause-acpica":                  true,
	"bsd-3-clause-attribution":             true,
	"bsd-3-clause-clear":                   true,
	"bsd-3-clause-flex":                    true,
	"bsd-3-clause-hp":                      true,
	"bsd-3-clause-lbnl":                    true,
	"bsd-3-clause-modification":            true,
	"bsd-3-clause-no-military-license":     true,
	"bsd-3-clause-no-nuclear-license":      true,
	"bsd-3-clause-no-nuclear-license-2014": true,
	"bsd-3-clause-no-nuclear-warranty":     true,
	"bsd-3-clause-open-mpi":                true,
	"bsd-3-clause-sun":                     true,
	"bsd-3-clause-tso":                     true,
	"bsd-4-clause":                         true,
	"bsd-4-clause-shortened":               true,
	"bsd-4-clause-uc":                      true,
	"bsd-4.3reno":                          true,
	"bsd-4.3tahoe":                         true,
	"bsd-advertising-acknowledgement":      true,
	"bsd-attribution-hpnd-disclaimer":      true,
	"bsd-inferno-nettverk":                 true,
	"bsd-mark-modifications":               true,
	"bsd-protection":                       true,
	"bsd-source-beginning-file":            true,
	"bsd-source-code":                      true,
	"bsd-systemics":                        true,
	"bsd-systemics-w3works":                true,
	"bsl-1.0":                              true,
	"buddy":                                true,
	"busl-1.1":                             true,
	"bzip2-1.0.5":                          true,
	"bzip2-1.0.6":                          true,
	"c-uda-1.0":                            true,
	"cal-1.0":                              true,
	"cal-1.0-combined-work-exception":      true,
	"caldera":                              true,
	"caldera-no-preamble":                  true,
	"capec-tou":                            true,
	"catharon":                             true,
	"catosl-1.1":                           true,
	"cc-by-1.0":                            true,
	"cc-by-2.0":                            true,
	"cc-by-2.5":                            true,
	"cc-by-2.5-au":                         true,
	"cc-by-3.0":                            true,
	"cc-by-3.0-at":                         true,
	"cc-by-3.0-au":                         true,
	"cc-by-3.0-de":                         true,
	"cc-by-3.0-igo":                        true,
	"cc-by-3.0-nl":                         true,
	"cc-by-3.0-us":                         true,
	"cc-by-4.0":                            true,
	"cc-by-nc-1.0":                         true,
	"cc-by-nc-2.0":                         true,
	"cc-by-nc-2.5":                         true,
	"cc-by-nc-3.0":                         true,
	"cc-by-nc-3.0-de":                      true,
	"cc-by-nc-4.0":                         true,
	"cc-by-nc-nd-1.0":                      true,
	"cc-by-nc-nd-2.0":                      true,
	"cc-by-nc-nd-2.5":                      true,
	"cc-by-nc-nd-3.0":                      true,
	"cc-by-nc-nd-3.0-de":                   true,
	"cc-by-nc-nd-3.0-igo":                  true,
	"cc-by-nc-nd-4.0":                      true,
	"cc-by-nc-sa-1.0":                      true,
	"cc-by-nc-sa-2.0":                      true,
	"cc-by-nc-sa-2.0-de":                   true,
	"cc-by-nc-sa-2.0-fr":                   true,
	"cc-by-nc-sa-2.0-uk":                   true,
	"cc-by-nc-sa-2.5":                      true,
	"cc-by-nc-sa-3.0":                      true,
	"cc-by-nc-sa-3.0-de":                   true,
	"cc-by-nc-sa-3.0-igo":                  true,
	"cc-by-nc-sa-4.0":                      true,
	"cc-by-nd-1.0":                         true,
	"cc-by-nd-2.0":                         true,
	"cc-by-nd-2.5":                         true,
	"cc-by-nd-3.0":                         true,
	"cc-by-nd-3.0-de":                      true,
	"cc-by-nd-4.0":                         true,
	"cc-by-sa-1.0":                         true,
	"cc-by-sa-2.0":                         true,
	"cc-by-sa-2.0-uk":                      true,
	"cc-by-sa-2.1-jp":                      true,
	"cc-by-sa-2.5":                         true,
	"cc-by-sa-3.0":                         true,
	"cc-by-sa-3.0-at":                      true,
	"cc-by-sa-3.0-de":                      true,
	"cc-by-sa-3.0-igo":                     true,
	"cc-by-sa-4.0":                         true,
	"cc-pddc":                              true,
	"cc-pdm-1.0":                           true,
	"cc-sa-1.0":                            true,
	"cc0-1.0":                              true,
	"cddl-1.0":                             true,
	"cddl-1.1":                             true,
	"cdl-1.0":                              true,
	"cdla-permissive-1.0":                  true,
	"cdla-permissive-2.0":                  true,
	"cdla-sharing-1.0":                     true,
	"cecill-1.0":                           true,
	"cecill-1.1":                           true,
	"cecill-2.0":                           true,
	"cecill-2.1":                           true,
	"cecill-b":                             true,
	"cecill-c":                             true,
	"cern-ohl-1.1":                         true,
	"cern-ohl-1.2":                         true,
	"cern-ohl-p-2.0":                       true,
	"cern-ohl-s-2.0":                       true,
	"cern-ohl-w-2.0":                       true,
	"cfitsio":                              true,
	"check-cvs":                            true,
	"checkmk":                              true,
	"clartistic":                           true,
	"clips":                                true,
	"cmu-mach":                             true,
	"cmu-mach-nodoc":                       true,
	"cnri-jython":                          true,
	"cnri-python":                          true,
	"cnri-python-gpl-compatible":           true,
	"coil-1.0":                             true,
	"community-spec-1.0":                   true,
	"condor-1.1":                           true,
	"copyleft-next-0.3.0":                  true,
	"copyleft-next-0.3.1":                  true,
	"cornell-lossless-jpeg":                true,
	"cpal-1.0":                             true,
	"cpl-1.0":                              true,
	"cpol-1.02":                            true,
	"cronyx":                               true,
	"crossword":                            true,
	"cryptoswift":                          true,
	"crystalstacker":                       true,
	"cua-opl-1.0":                          true,
	"cube":                                 true,
	"curl":                                 true,
	"cve-tou":                              true,
	"d-fsl-1.0":                            true,
	"dec-3-clause":                         true,
	"diffmark":                             true,
	"dl-de-by-2.0":                         true,
	"dl-de-zero-2.0":                       true,
	"doc":                                  true,
	"docbook-dtd":                          true,
	"docbook-schema":                       true,
	"docbook-stylesheet":                   true,
	"docbook-xml":                          true,
	"dotseqn":                              true,
	"drl-1.0":                              true,
	"drl-1.1":                              true,
	"dsdp":                                 true,
	"dtoa":                                 true,
	"dvipdfm":                              true,
	"ecl-1.0":                              true,
	"ecl-2.0":                              true,
	"ecos-2.0":                             true,
	"efl-1.0":                              true,
	"efl-2.0":                              true,
	"egenix":                               true,
	"elastic-2.0":                          true,
	"entessa":                              true,
	"epics":                                true,
	"epl-1.0":                              true,
	"epl-2.0":                              true,
	"erlpl-1.1":                            true,
	"esa-pl-permissive-2.4":                true,
	"esa-pl-strong-copyleft-2.4":           true,
	"esa-pl-weak-copyleft-2.4":             true,
	"etalab-2.0":                           true,
	"eudatagrid":                           true,
	"eupl-1.0":                             true,
	"eupl-1.1":                             true,
	"eupl-1.2":                             true,
	"eurosym":                              true,
	"fair":                                 true,
	"fbm":                                  true,
	"fdk-aac":                              true,
	"ferguson-twofish":                     true,
	"frameworx-1.0":                        true,
	"freebsd-doc":                          true,
	"freeimage":                            true,
	"fsfap":                                true,
	"fsfap-no-warranty-disclaimer":         true,
	"fsful":                                true,
	"fsfullr":                              true,
	"fsfullrsd":                            true,
	"fsfullrwd":                            true,
	"fsl-1.1-alv2":                         true,
	"fsl-1.1-mit":                          true,
	"ftl":                                  true,
	"furuseth":                             true,
	"fwlw":                                 true,
	"game-programming-gems":                true,
	"gcr-docs":                             true,
	"gd":                                   true,
	"generic-xts":                          true,
	"gfdl-1.1":                             true,
	"gfdl-1.1-invariants-only":             true,
	"gfdl-1.1-invariants-or-later":         true,
	"gfdl-1.1-no-invariants-only":          true,
	"gfdl-1.1-no-invariants-or-later":      true,
	"gfdl-1.1-only":                        true,
	"gfdl-1.1-or-later":                    true,
	"gfdl-1.2":                             true,
	"gfdl-1.2-invariants-only":             true,
	"gfdl-1.2-invariants-or-later":         true,
	"gfdl-1.2-no-invariants-only":          true,
	"gfdl-1.2-no-invariants-or-later":      true,
	"gfdl-1.2-only":                        true,
	"gfdl-1.2-or-later":                    true,
	"gfdl-1.3":                             true,
	"gfdl-1.3-invariants-only":             true,
	"gfdl-1.3-invariants-or-later":         true,
	"gfdl-1.3-no-invariants-only":          true,
	"gfdl-1.3-no-invariants-or-later":      true,
	"gfdl-1.3-only":                        true,
	"gfdl-1.3-or-later":                    true,
	"giftware":                             true,
	"gl2ps":                                true,
	"glide":                                true,
	"glulxe":                               true,
	"glwtpl":                               true,
	"gnuplot":                              true,
	"gpl-1.0":                              true,
	"gpl-1.0+":                             true,
	"gpl-1.0-only":                         true,
	"gpl-1.0-or-later":                     true,
	"gpl-2.0":                              true,
	"gpl-2.0+":                             true,
	"gpl-2.0-only":                         true,
	"gpl-2.0-or-later":                     true,
	"gpl-2.0-with-autoconf-exception":      true,
	"gpl-2.0-with-bison-exception":         true,
	"gpl-2.0-with-classpath-exception":     true,
	"gpl-2.0-with-font-exception":          true,
	"gpl-2.0-with-gcc-exception":           true,
	"gpl-3.0":                              true,
	"gpl-3.0+":                             true,
	"gpl-3.0-only":                         true,
	"gpl-3.0-or-later":                     true,
	"gpl-3.0-with-autoconf-exception":      true,
	"gpl-3.0-with-gcc-exception":           true,
	"graphics-gems":                        true,
	"gsoap-1.3b":                           true,
	"gtkbook":                              true,
	"gutmann":                              true,
	"haskellreport":                        true,
	"hdf5":                                 true,
	"hdparm":                               true,
	"hidapi":                               true,
	"hippocratic-2.1":                      true,
	"hp-1986":                              true,
	"hp-1989":                              true,
	"hpnd":                                 true,
	"hpnd-dec":                             true,
	"hpnd-doc":                             true,
	"hpnd-doc-sell":                        true,
	"hpnd-export-us":                       true,
	"hpnd-export-us-acknowledgement":       true,
	"hpnd-export-us-modify":                true,
	"hpnd-export2-us":                      true,
	"hpnd-fenneberg-livingston":            true,
	"hpnd-inria-imag":                      true,
	"hpnd-intel":                           true,
	"hpnd-kevlin-henney":                   true,
	"hpnd-markus-kuhn":                     true,
	"hpnd-merchantability-variant":         true,
	"hpnd-mit-disclaimer":                  true,
	"hpnd-netrek":                          true,
	"hpnd-pbmplus":                         true,
	"hpnd-sell-mit-disclaimer-xserver":     true,
	"hpnd-sell-regexpr":                    true,
	"hpnd-sell-variant":                    true,
	"hpnd-sell-variant-critical-systems":   true,
	"hpnd-sell-variant-mit-disclaimer":     true,
	"hpnd-sell-variant-mit-disclaimer-rev": true,
	"hpnd-smc":                             true,
	"hpnd-uc":                              true,
	"hpnd-uc-export-us":                    true,
	"htmltidy":                             true,
	"hyphen-bulgarian":                     true,
	"ibm-pibs":                             true,
	"icu":                                  true,
	"iec-code-components-eula":             true,
	"ijg":                                  true,
	"ijg-short":                            true,
	"imagemagick":                          true,
	"imatix":                               true,
	"imlib2":                               true,
	"info-zip":                             true,
	"inner-net-2.0":                        true,
	"innosetup":                            true,
	"intel":                                true,
	"intel-acpi":                           true,
	"interbase-1.0":                        true,
	"ipa":                                  true,
	"ipl-1.0":                              true,
	"isc":                                  true,
	"isc-veillard":                         true,
	"iso-permission":                       true,
	"jam":                                  true,
	"jasper-2.0":                           true,
	"jove":                                 true,
	"jpl-image":                            true,
	"jpnic":                                true,
	"json":                                 true,
	"kastrup":                              true,
	"kazlib":                               true,
	"knuth-ctan":                           true,
	"lal-1.2":                              true,
	"lal-1.3":                              true,
	"latex2e":                              true,
	"latex2e-translated-notice":            true,
	"leptonica":                            true,
	"lgpl-2.0":                             true,
	"lgpl-2.0+":                            true,
	"lgpl-2.0-only":                        true,
	"lgpl-2.0-or-later":                    true,
	"lgpl-2.1":                             true,
	"lgpl-2.1+":                            true,
	"lgpl-2.1-only":                        true,
	"lgpl-2.1-or-later":                    true,
	"lgpl-3.0":                             true,
	"lgpl-3.0+":                            true,
	"lgpl-3.0-only":                        true,
	"lgpl-3.0-or-later":                    true,
	"lgpllr":                               true,
	"libpng":                               true,
	"libpng-1.6.35":                        true,
	"libpng-2.0":                           true,
	"libselinux-1.0":                       true,
	"libtiff":                              true,
	"libutil-david-nugent":                 true,
	"liliq-p-1.1":                          true,
	"liliq-r-1.1":                          true,
	"liliq-rplus-1.1":                      true,
	"linux-man-pages-1-para":               true,
	"linux-man-pages-copyleft":             true,
	"linux-man-pages-copyleft-2-para":      true,
	"linux-man-pages-copyleft-var":         true,
	"linux-openib":                         true,
	"loop":                                 true,
	"lpd-document":                         true,
	"lpl-1.0":                              true,
	"lpl-1.02":                             true,
	"lppl-1.0":                             true,
	"lppl-1.1":                             true,
	"lppl-1.2":                             true,
	"lppl-1.3a":                            true,
	"lppl-1.3c":                            true,
	"lsof":                                 true,
	"lucida-bitmap-fonts":                  true,
	"lzma-sdk-9.11-to-9.20":                true,
	"lzma-sdk-9.22":                        true,
	"mackerras-3-clause":                   true,
	"mackerras-3-clause-acknowledgment":    true,
	"magaz":                                true,
	"mailprio":                             true,
	"makeindex":                            true,
	"man2html":                             true,
	"martin-birgmeier":                     true,
	"mcphee-slideshow":                     true,
	"metamail":                             true,
	"minpack":                              true,
	"mips":                                 true,
	"miros":                                true,
	"mit":                                  true,
	"mit-0":                                true,
	"mit-advertising":                      true,
	"mit-click":                            true,
	"mit-cmu":                              true,
	"mit-enna":                             true,
	"mit-feh":                              true,
	"mit-festival":                         true,
	"mit-khronos-old":                      true,
	"mit-modern-variant":                   true,
	"mit-open-group":                       true,
	"mit-stk":                              true,
	"mit-testregex":                        true,
	"mit-wu":                               true,
	"mitnfa":                               true,
	"mmixware":                             true,
	"mmpl-1.0.1":                           true,
	"motosoto":                             true,
	"mpeg-ssg":                             true,
	"mpi-permissive":                       true,
	"mpich2":                               true,
	"mpl-1.0":                              true,
	"mpl-1.1":                              true,
	"mpl-2.0":                              true,
	"mpl-2.0-no-copyleft-exception":        true,
	"mplus":                                true,
	"ms-lpl":                               true,
	"ms-pl":                                true,
	"ms-rl":                                true,
	"mtll":                                 true,
	"mulanpsl-1.0":                         true,
	"mulanpsl-2.0":                         true,
	"multics":                              true,
	"mup":                                  true,
	"mvt-1.1":                              true,
	"naist-2003":                           true,
	"nasa-1.3":                             true,
	"naumen":                               true,
	"nbpl-1.0":                             true,
	"ncbi-pd":                              true,
	"ncgl-uk-2.0":                          true,
	"ncl":                                  true,
	"ncsa":                                 true,
	"net-snmp":                             true,
	"netcdf":                               true,
	"newsletr":                             true,
	"ngpl":                                 true,
	"ngrep":                                true,
	"nicta-1.0":                            true,
	"nist-pd":                              true,
	"nist-pd-fallback":                     true,
	"nist-pd-tnt":                          true,
	"nist-software":                        true,
	"nlod-1.0":                             true,
	"nlod-2.0":                             true,
	"nlpl":                                 true,
	"nokia":                                true,
	"nosl":                                 true,
	"noweb":                                true,
	"npl-1.0":                              true,
	"npl-1.1":                              true,
	"nposl-3.0":                            true,
	"nrl":                                  true,
	"ntia-pd":                              true,
	"ntp":                                  true,
	"ntp-0":                                true,
	"nunit":                                true,
	"o-uda-1.0":                            true,
	"oar":                                  true,
	"occt-pl":                              true,
	"oclc-2.0":                             true,
	"odbl-1.0":                             true,
	"odc-by-1.0":                           true,
	"offis":                                true,
	"ofl-1.0":                              true,
	"ofl-1.0-no-rfn":                       true,
	"ofl-1.0-rfn":                          true,
	"ofl-1.1":                              true,
	"ofl-1.1-no-rfn":                       true,
	"ofl-1.1-rfn":                          true,
	"ogc-1.0":                              true,
	"ogdl-taiwan-1.0":                      true,
	"ogl-canada-2.0":                       true,
	"ogl-uk-1.0":                           true,
	"ogl-uk-2.0":                           true,
	"ogl-uk-3.0":                           true,
	"ogtsl":                                true,
	"oldap-1.1":                            true,
	"oldap-1.2":                            true,
	"oldap-1.3":                            true,
	"oldap-1.4":                            true,
	"oldap-2.0":                            true,
	"oldap-2.0.1":                          true,
	"oldap-2.1":                            true,
	"oldap-2.2":                            true,
	"oldap-2.2.1":                          true,
	"oldap-2.2.2":                          true,
	"oldap-2.3":                            true,
	"oldap-2.4":                            true,
	"oldap-2.5":                            true,
	"oldap-2.6":                            true,
	"oldap-2.7":                            true,
	"oldap-2.8":                            true,
	"olfl-1.3":                             true,
	"oml":                                  true,
	"openmdw-1.0":                          true,
	"openpbs-2.3":                          true,
	"openssl":                              true,
	"openssl-standalone":                   true,
	"openvision":                           true,
	"opl-1.0":                              true,
	"opl-uk-3.0":                           true,
	"opubl-1.0":                            true,
	"osc-1.0":                              true,
	"oset-pl-2.1":                          true,
	"osl-1.0":                              true,
	"osl-1.1":                              true,
	"osl-2.0":                              true,
	"osl-2.1":                              true,
	"osl-3.0":                              true,
	"ossp":                                 true,
	"padl":                                 true,
	"paratype-free-font-1.3":               true,
	"parity-6.0.0":                         true,
	"parity-7.0.0":                         true,
	"pddl-1.0":                             true,
	"php-3.0":                              true,
	"php-3.01":                             true,
	"pixar":                                true,
	"pkgconf":                              true,
	"plexus":                               true,
	"pnmstitch":                            true,
	"polyform-noncommercial-1.0.0":         true,
	"polyform-small-business-1.0.0":        true,
	"postgresql":                           true,
	"ppl":                                  true,
	"psf-2.0":                              true,
	"psfrag":                               true,
	"psutils":                              true,
	"python-2.0":                           true,
	"python-2.0.1":                         true,
	"python-ldap":                          true,
	"qhull":                                true,
	"qpl-1.0":                              true,
	"qpl-1.0-inria-2004":                   true,
	"radvd":                                true,
	"rdisc":                                true,
	"rhecos-1.1":                           true,
	"rpl-1.1":                              true,
	"rpl-1.5":                              true,
	"rpsl-1.0":                             true,
	"rsa-md":                               true,
	"rscpl":                                true,
	"ruby":                                 true,
	"ruby-pty":                             true,
	"sax-pd":                               true,
	"sax-pd-2.0":                           true,
	"saxpath":                              true,
	"scea":                                 true,
	"schemereport":                         true,
	"sendmail":                             true,
	"sendmail-8.23":                        true,
	"sendmail-open-source-1.1":             true,
	"sgi-b-1.0":                            true,
	"sgi-b-1.1":                            true,
	"sgi-b-2.0":                            true,
	"sgi-opengl":                           true,
	"sgmlug-pm":                            true,
	"sgp4":                                 true,
	"shl-0.5":                              true,
	"shl-0.51":                             true,
	"simpl-2.0":                            true,
	"sissl":                                true,
	"sissl-1.2":                            true,
	"sl":                                   true,
	"sleepycat":                            true,
	"smail-gpl":                            true,
	"smlnj":                                true,
	"smppl":                                true,
	"snia":                                 true,
	"snprintf":                             true,
	"sofa":                                 true,
	"softsurfer":                           true,
	"soundex":                              true,
	"spencer-86":                           true,
	"spencer-94":                           true,
	"spencer-99":                           true,
	"spl-1.0":                              true,
	"ssh-keyscan":                          true,
	"ssh-openssh":                          true,
	"ssh-short":                            true,
	"ssleay-standalone":                    true,
	"sspl-1.0":                             true,
	"standardml-nj":                        true,
	"sugarcrm-1.1.3":                       true,
	"sul-1.0":                              true,
	"sun-ppp":                              true,
	"sun-ppp-2000":                         true,
	"sunpro":                               true,
	"swl":                                  true,
	"swrule":                               true,
	"symlinks":                             true,
	"tapr-ohl-1.0":                         true,
	"tcl":                                  true,
	"tcp-wrappers":                         true,
	"tekhvc":                               true,
	"termreadkey":                          true,
	"tgppl-1.0":                            true,
	"thirdeye":                             true,
	"threeparttable":                       true,
	"tmate":                                true,
	"torque-1.1":                           true,
	"tosl":                                 true,
	"tpdl":                                 true,
	"tpl-1.0":                              true,
	"trustedqsl":                           true,
	"ttwl":                                 true,
	"ttyp0":                                true,
	"tu-berlin-1.0":                        true,
	"tu-berlin-2.0":                        true,
	"ubuntu-font-1.0":                      true,
	"ucar":                                 true,
	"ucl-1.0":                              true,
	"ulem":                                 true,
	"umich-merit":                          true,
	"unicode-3.0":                          true,
	"unicode-dfs-2015":                     true,
	"unicode-dfs-2016":                     true,
	"unicode-tou":                          true,
	"unixcrypt":                            true,
	"unlicense":                            true,
	"unlicense-libtelnet":                  true,
	"unlicense-libwhirlpool":               true,
	"unrar":                                true,
	"upl-1.0":                              true,
	"urt-rle":                              true,
	"vim":                                  true,
	"vixie-cron":                           true,
	"vostrom":                              true,
	"vsl-1.0":                              true,
	"w3c":                                  true,
	"w3c-19980720":                         true,
	"w3c-20150513":                         true,
	"w3m":                                  true,
	"watcom-1.0":                           true,
	"widget-workshop":                      true,
	"wordnet":                              true,
	"wsuipa":                               true,
	"wtfnmfpl":                             true,
	"wtfpl":                                true,
	"wwl":                                  true,
	"wxwindows":                            true,
	"x11":                                  true,
	"x11-distribute-modifications-variant": true,
	"x11-no-permit-persons":                true,
	"x11-swapped":                          true,
	"xdebug-1.03":                          true,
	"xerox":                                true,
	"xfig":                                 true,
	"xfree86-1.1":                          true,
	"xinetd":                               true,
	"xkeyboard-config-zinoviev":            true,
	"xlock":                                true,
	"xnet":                                 true,
	"xpp":                                  true,
	"xskat":                                true,
	"xzoom":                                true,
	"ypl-1.0":                              true,
	"ypl-1.1":                              true,
	"zed":                                  true,
	"zeeff":                                true,
	"zend-2.0":                             true,
	"zimbra-1.3":                           true,
	"zimbra-1.4":                           true,
	"zlib":                                 true,
	"zlib-acknowledgement":                 true,
	"zpl-1.1":                              true,
	"zpl-2.0":                              true,
	"zpl-2.1":                              true,
}
